"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initComponentTesting = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const inquirer_1 = __importDefault(require("inquirer"));
const cli_highlight_1 = __importDefault(require("cli-highlight"));
const guessTemplate_1 = require("./templates/guessTemplate");
const installFrameworkAdapter_1 = require("./installFrameworkAdapter");
const babelTransform_1 = require("./babel/babelTransform");
const utils_1 = require("../utils");
async function injectOrShowConfigCode(injectFn, { code, filePath, fallbackFileMessage, language, }) {
    const fileExists = fs_extra_1.default.existsSync(filePath);
    const readableFilePath = fileExists ? path_1.default.relative(process.cwd(), filePath) : fallbackFileMessage;
    const printCode = () => {
        console.log();
        console.log(cli_highlight_1.default(code, { language }));
        console.log();
    };
    const printSuccess = () => {
        console.log(`✅  ${chalk_1.default.bold.green(readableFilePath)} was updated with the following config:`);
        printCode();
    };
    const printFailure = () => {
        console.log(`❌  ${chalk_1.default.bold.red(readableFilePath)} was not updated automatically. Please add the following config manually: `);
        printCode();
    };
    if (!fileExists) {
        printFailure();
        return;
    }
    // something get completely wrong when using babel or something. Print error message.
    const injected = await injectFn().catch(() => false);
    injected ? printSuccess() : printFailure();
}
async function injectAndShowCypressJsonConfig(cypressJsonPath, componentFolder) {
    const configToInject = {
        componentFolder,
        testFiles: '**/*.spec.{js,ts,jsx,tsx}',
    };
    async function autoInjectCypressJson() {
        const currentConfig = JSON.parse(await fs_extra_1.default.readFile(cypressJsonPath, { encoding: 'utf-8' }));
        await fs_extra_1.default.writeFile(cypressJsonPath, JSON.stringify({
            ...currentConfig,
            ...configToInject,
        }, null, 2));
        return true;
    }
    await injectOrShowConfigCode(autoInjectCypressJson, {
        code: JSON.stringify(configToInject, null, 2),
        language: 'js',
        filePath: cypressJsonPath,
        fallbackFileMessage: 'cypress.json config file',
    });
}
async function injectAndShowPluginConfig(template, { templatePayload, pluginsFilePath, cypressProjectRoot, }) {
    const ast = template.getPluginsCodeAst(templatePayload, { cypressProjectRoot });
    await injectOrShowConfigCode(() => babelTransform_1.injectPluginsCode(pluginsFilePath, ast), {
        code: await babelTransform_1.getPluginsSourceExample(ast),
        language: 'js',
        filePath: pluginsFilePath,
        fallbackFileMessage: 'plugins file (https://docs.cypress.io/guides/core-concepts/writing-and-organizing-tests.html#Plugin-files)',
    });
}
async function initComponentTesting({ config, useYarn, cypressConfigPath }) {
    var _a;
    const cypressProjectRoot = path_1.default.resolve(cypressConfigPath, '..');
    const framework = await installFrameworkAdapter_1.installFrameworkAdapter(cypressProjectRoot, { useYarn });
    const { possibleTemplates, defaultTemplate, defaultTemplateName, templatePayload, } = await guessTemplate_1.guessTemplate(framework, cypressProjectRoot);
    const pluginsFilePath = path_1.default.resolve(cypressProjectRoot, (_a = config.pluginsFile) !== null && _a !== void 0 ? _a : './cypress/plugins/index.js');
    const templateChoices = Object.keys(possibleTemplates).sort((key) => {
        return key === defaultTemplateName ? -1 : 0;
    });
    const { chosenTemplateName, componentFolder, } = await inquirer_1.default.prompt([
        {
            type: 'list',
            name: 'chosenTemplateName',
            choices: templateChoices,
            default: defaultTemplate ? 0 : undefined,
            message: (defaultTemplate === null || defaultTemplate === void 0 ? void 0 : defaultTemplate.message)
                ? `${defaultTemplate === null || defaultTemplate === void 0 ? void 0 : defaultTemplate.message}\n\n Press ${chalk_1.default.inverse(' Enter ')} to continue with ${chalk_1.default.green(defaultTemplateName)} configuration or select another template from the list:`
                : 'We were not able to automatically determine which framework or bundling tool you are using. Please choose one from the list:',
        },
        {
            type: 'input',
            name: 'componentFolder',
            filter: (input) => input.trim(),
            validate: (input) => {
                return input === '' || !/^[a-zA-Z].*/.test(input)
                    ? `Directory "${input}" is invalid`
                    : true;
            },
            message: 'Which folder would you like to use for your component tests?',
            default: (answers) => {
                return possibleTemplates[answers.chosenTemplateName].recommendedComponentFolder;
            },
        },
    ]);
    const chosenTemplate = possibleTemplates[chosenTemplateName];
    console.log();
    console.log(`Installing required dependencies`);
    console.log();
    for (const dependency of chosenTemplate.dependencies) {
        await utils_1.installDependency(dependency, { useYarn });
    }
    console.log();
    console.log(`Let's setup everything for component testing with ${chalk_1.default.cyan(chosenTemplateName)}:`);
    console.log();
    await injectAndShowCypressJsonConfig(cypressConfigPath, componentFolder);
    await injectAndShowPluginConfig(chosenTemplate, {
        templatePayload,
        pluginsFilePath,
        cypressProjectRoot,
    });
    if (chosenTemplate.printHelper) {
        chosenTemplate.printHelper();
    }
    console.log(`Find examples of component tests for ${chalk_1.default.green(chosenTemplateName)} in ${chalk_1.default.underline(chosenTemplate.getExampleUrl({ componentFolder }))}.`);
    if (framework === 'react') {
        console.log();
        console.log(`Docs for different recipes of bundling tools: ${chalk_1.default.bold.underline('https://github.com/cypress-io/cypress/tree/develop/npm/react/docs/recipes.md')}`);
    }
    // render delimiter
    console.log();
    console.log(new Array(process.stdout.columns).fill('═').join(''));
    console.log();
}
exports.initComponentTesting = initComponentTesting;
