"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const fs_1 = __importDefault(require("fs"));
const find_up_1 = __importDefault(require("find-up"));
const chalk_1 = __importDefault(require("chalk"));
const util_1 = __importDefault(require("util"));
const inquirer_1 = __importDefault(require("inquirer"));
const init_component_testing_1 = require("./component-testing/init-component-testing");
const child_process_1 = require("child_process");
const findPackageJson_1 = require("./findPackageJson");
const installCypress_1 = require("./installCypress");
async function getGitStatus() {
    const execAsync = util_1.default.promisify(child_process_1.exec);
    try {
        let { stdout } = await execAsync(`git status --porcelain`);
        console.log(stdout);
        return stdout.trim();
    }
    catch (e) {
        return '';
    }
}
async function shouldUseYarn() {
    const execAsync = util_1.default.promisify(child_process_1.exec);
    return execAsync('yarn --version')
        .then(() => true)
        .catch(() => false);
}
function shouldUseTypescript() {
    return findPackageJson_1.scanFSForAvailableDependency(process.cwd(), { typescript: '*' });
}
async function askForComponentTesting() {
    const { shouldSetupComponentTesting } = await inquirer_1.default.prompt({
        type: 'confirm',
        name: 'shouldSetupComponentTesting',
        message: `Do you want to setup ${chalk_1.default.cyan('component testing')}? ${chalk_1.default.grey('You can do this later by rerunning this command')}.`,
    });
    return shouldSetupComponentTesting;
}
function printCypressCommandsHelper(options) {
    const printCommand = (command, description) => {
        const displayedRunner = options.useYarn ? 'yarn' : 'npx';
        console.log();
        console.log(chalk_1.default.cyan(`  ${displayedRunner} ${command}`));
        console.log(`    ${description}`);
    };
    printCommand('cypress open', 'Opens cypress local development app.');
    printCommand('cypress run', 'Runs tests in headless mode.');
    if (options.shouldSetupComponentTesting) {
        printCommand('cypress open-ct', 'Opens cypress component-testing web app.');
        printCommand('cypress run-ct', 'Runs component testing in headless mode.');
    }
}
async function main({ useNpm, ignoreTs, setupComponentTesting, ignoreExamples }) {
    const rootPackageJsonPath = await find_up_1.default('package.json');
    const useYarn = useNpm === true ? false : await shouldUseYarn();
    const useTypescript = ignoreTs ? false : shouldUseTypescript();
    if (!rootPackageJsonPath) {
        console.log(`${chalk_1.default.bold.red(`It looks like you are running cypress installation wizard outside of npm module.`)}\nIf you would like to setup a new project for cypress tests please run the ${chalk_1.default.inverse(useNpm ? ' npm init ' : ' yarn init ')} first.`);
        process.exit(1);
    }
    const { name = 'unknown', version = '0.0.0' } = JSON.parse(fs_1.default.readFileSync(rootPackageJsonPath).toString());
    console.log(`Running ${chalk_1.default.green('cypress 🌲')} installation wizard for ${chalk_1.default.cyan(`${name}@${version}`)}`);
    const gitStatus = await getGitStatus();
    if (gitStatus) {
        console.error(`\n${chalk_1.default.bold.red('This repository has untracked files or uncommmited changes.')}\nThis command will ${chalk_1.default.cyan('make changes in the codebase')}, so please remove untracked files, stash or commit any changes, and try again.`);
        process.exit(1);
    }
    const { config, cypressConfigPath } = await installCypress_1.findInstalledOrInstallCypress({ useYarn, useTypescript, ignoreExamples });
    const shouldSetupComponentTesting = setupComponentTesting !== null && setupComponentTesting !== void 0 ? setupComponentTesting : await askForComponentTesting();
    if (shouldSetupComponentTesting) {
        await init_component_testing_1.initComponentTesting({ config, cypressConfigPath, useYarn });
    }
    console.log(`\n👍  Success! Cypress is installed and ready to run tests.`);
    printCypressCommandsHelper({ useYarn, shouldSetupComponentTesting });
    console.log(`\nHappy testing with ${chalk_1.default.green('cypress.io')} 🌲\n`);
}
exports.main = main;
